/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.infofirma.service.impl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import javax.annotation.PostConstruct;
import javax.naming.OperationNotSupportedException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Required;

import es.mpt.dsic.eeutil.client.model.ApplicationLogin;
import es.mpt.dsic.eeutil.client.model.InSideException;
import es.mpt.dsic.eeutil.client.model.InformacionFirma;
import es.mpt.dsic.eeutil.client.model.OpcionesObtenerInformacionFirma;
import es.mpt.dsic.eeutil.client.service.EeUtilService;
import es.mpt.dsic.eeutil.client.service.EeUtilServiceImplService;
import es.mpt.dsic.infofirma.converter.InfoFirmaConverter;
import es.mpt.dsic.infofirma.model.FirmaElectronica;
import es.mpt.dsic.infofirma.model.InfoFirmaElectronica;
import es.mpt.dsic.infofirma.model.OpcionesInfoFirma;
import es.mpt.dsic.infofirma.service.InfoFirmaService;
import es.mpt.dsic.infofirma.service.exception.InfoFirmaServiceException;
import es.mpt.dsic.infofirma.service.exception.InfoFirmaServiceNoEsFirmaException;

public class InfoFirmaServiceImpl implements InfoFirmaService {
	
	protected static final Log logger = LogFactory.getLog(InfoFirmaServiceImpl.class);
	
	private boolean activo;
	private static String ACTIVO = "S";
	
	private Properties properties;
	private EeUtilService port;
	private ApplicationLogin applicationLogin;
	
	@PostConstruct
	public void configureInfoFirma() {
		String infoFirmaActivo = properties.getProperty("infofirma.activo");
		
		if (!ACTIVO.contentEquals(infoFirmaActivo)) {
			logger.info("El WS de INFOFIRMA para la obtención de información de firmas no está activo");
			activo = false;
		} else {
			URL url = null;
			String urlInfoFirma = null;
			try {
				urlInfoFirma = properties.getProperty("infofirma.url");
				logger.debug(String.format("El WS de INFOFIRMA se encuentra en %s", urlInfoFirma));
				url = new URL (urlInfoFirma);
			} catch (MalformedURLException me) {
				logger.error("No se puede crear la URL del servicio de infofirma " + urlInfoFirma, me);
			}
			
			EeUtilServiceImplService ss = new EeUtilServiceImplService(url);	
			
		    port = ss.getEeUtilServiceImplPort();  
		    
		    applicationLogin = new ApplicationLogin();
		    applicationLogin.setIdaplicacion(properties.getProperty("infofirma.idaplicacion"));
		    applicationLogin.setPassword(properties.getProperty("infofirma.password"));
		    
		    logger.debug(String.format("Utilizando para infofirma idaplicacion/password : %s/%s", properties.getProperty("infofirma.idaplicacion"), properties.getProperty("infofirma.password")));
		    activo = true;
		}
	    
	}
	
	
	
	/**
	 * Obtiene la información de una firma electrónica.
	 * @throws InfoFirmaServiceNoEsFirmaException cuando el contenido aportado no se corresponde con una firma electrónica en un formato de los admitidos.
	 * @throws InfoFirmaServiceException cuando no se pueda obtener la información de la firma
	 */
	public InfoFirmaElectronica getInfoFirma (FirmaElectronica firma, OpcionesInfoFirma opciones) throws InfoFirmaServiceNoEsFirmaException, InfoFirmaServiceException {
		if (!activo) {
			throw new InfoFirmaServiceException ("El WS de INFOFIRMA para la obtención de información de firmas no está activo");
		}
		
		if (firma == null || firma.getFirma() == null) {
			throw new InfoFirmaServiceException (new IllegalArgumentException ("La firma no puede ser nula"));
		}
		
		try {
			OpcionesObtenerInformacionFirma opcionesWS = null;
			if (opciones == null) {
				opcionesWS = new OpcionesObtenerInformacionFirma();
				opcionesWS.setObtenerDatosFirmados(true);
				opcionesWS.setObtenerFirmantes(true);
				opcionesWS.setObtenerTipoFirma(true);
			} else {
				opcionesWS = InfoFirmaConverter.OpcionesInfoFirmaToOpcionesObtenerInformacionFirma(opciones);
			}
			
			
			InformacionFirma infoFirma = port.obtenerInformacionFirma(applicationLogin, firma.getFirma(), opcionesWS);
			
			if (!infoFirma.isEsFirma()) {
				throw new InfoFirmaServiceNoEsFirmaException ("El contenido aportado no se corresponde con una firma electrónica de un formato de los admitidos");
			}
			
			InfoFirmaElectronica infoFirmaReturn = InfoFirmaConverter.informacionFirmaToInfoFirmaElectronica(infoFirma);
			
			return infoFirmaReturn;
			
			
		} catch (InSideException e) {
			if (e.getFaultInfo().getCodigo().equalsIgnoreCase("COD_0001") || e.getFaultInfo().getCodigo().equalsIgnoreCase("COD_0003")) {
				throw new InfoFirmaServiceNoEsFirmaException ("El contenido aportado no se corresponde con una firma electrónica de un formato de los admitidos");
			} else {
				logger.error("El servicio de obtención de información de firma está devolviendo un error: CODIGO: " + e.getFaultInfo().getCodigo() +  " DESCRIPCION: " + e.getFaultInfo().getDescripcion());
				throw new InfoFirmaServiceException ("El servicio de obtención de información de firma está devolviendo un error");
			}
			
		} catch (Throwable t) {
			logger.error("Excepción en la llamada al WS de obtención de Información de Firma", t);
			throw new InfoFirmaServiceException ("Excepción en la llamada al WS de obtención de Información de Firma", t);
		}
		
	}
	
	public Properties getProperties() {
		return properties;
	}

	@Required
	public void setProperties(Properties properties) {
		this.properties = properties;
	}
	

}
