/* Copyright (C) 2012-13 MINHAP, Gobierno de España
   This program is licensed and may be used, modified and redistributed under the terms
   of the European Public License (EUPL), either version 1.1 or (at your
   option) any later version as soon as they are approved by the European Commission.
   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
   or implied. See the License for the specific language governing permissions and
   more details.
   You should have received a copy of the EUPL1.1 license
   along with this program; if not, you may find it at
   http://joinup.ec.europa.eu/software/page/eupl/licence-eupl */

package es.mpt.dsic.inside.service.store.impl.hibernate;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.classic.Session;
import org.hibernate.criterion.Property;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.ClassUtils;

import es.mpt.dsic.inside.model.busqueda.consulta.ConsultaInside;
import es.mpt.dsic.inside.model.busqueda.resultado.DocumentoResultadoBusquedaInside;
import es.mpt.dsic.inside.model.busqueda.resultado.ExpedienteResultadoBusquedaInside;
import es.mpt.dsic.inside.model.busqueda.resultado.ItemResultadoBusquedaInside;
import es.mpt.dsic.inside.model.busqueda.resultado.ResultadoBusquedaInside;
import es.mpt.dsic.inside.model.objetos.ObjetoInside;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideAplicacion;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideVersion;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideVersionComparator;
import es.mpt.dsic.inside.model.objetos.ObjetoInsideVersionable;
import es.mpt.dsic.inside.model.objetos.documento.ObjetoDocumentoInside;
import es.mpt.dsic.inside.model.objetos.documento.metadatos.ObjetoDocumentoInsideMetadatos;
import es.mpt.dsic.inside.model.objetos.expediente.ObjetoExpedienteInside;
import es.mpt.dsic.inside.model.objetos.expediente.metadatos.ObjetoExpedienteInsideMetadatos;
import es.mpt.dsic.inside.service.object.definitions.InsideObjectDefinitionsContainer;
import es.mpt.dsic.inside.service.store.InsideServiceStore;
import es.mpt.dsic.inside.service.store.exception.InsideServiceStoreException;
import es.mpt.dsic.inside.service.store.exception.InsideStoreObjectAlreadyExistsException;
import es.mpt.dsic.inside.service.store.exception.InsideStoreObjectNotFoundException;
import es.mpt.dsic.inside.service.store.exception.InsideStoreObjectVersionNotFoundException;
import es.mpt.dsic.inside.service.store.impl.hibernate.converter.InsideServiceHibernateConverterBusqueda;
import es.mpt.dsic.inside.service.store.impl.hibernate.converter.InsideServiceStoreHibernateConverter;
import es.mpt.dsic.inside.service.store.impl.hibernate.converter.InsideServiceStoreHibernateConverterAplicacion;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.DocumentoInside;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.ExpedienteInside;
import es.mpt.dsic.inside.service.store.impl.hibernate.entity.InsideWsAplicacion;

public class InsideServiceStoreHibernateImpl <I extends ObjetoInside<?>, E extends Object> implements InsideServiceStore {
	
	protected static final Log logger = LogFactory.getLog(InsideServiceStore.class);
	
	private SessionFactory sessionFactory;
	
	private InsideServiceStoreHibernatePersister<I,E> persister;
	
	private InsideServiceStoreHibernateConverter<I,E> converter;
	
	@Autowired
	@Qualifier("InsideObjectDefinitions")
	private InsideObjectDefinitionsContainer objectDefinitions;

	@Override
	public <T extends ObjetoInside<?>> T storeObject(T object)
		throws InsideServiceStoreException, InsideStoreObjectAlreadyExistsException 
	{
		if(StringUtils.isBlank(object.getIdentificador())){
			throw new InsideServiceStoreException("El identicador no puede estár vacío");
		}
		
		try{
			getObject(object.getClass() , object.getIdentificador());
			throw new InsideStoreObjectAlreadyExistsException(object);
		}catch(InsideStoreObjectNotFoundException e){
			logger.debug("Almacenando objeto " + object.getClass() + " con identificador proporcionado " + object.getIdentificador());
		}
	
		persister.persistExpedienteOrDocumento(object);
		
		return object;
				
	}

	
	@Override
	public <T extends ObjetoInside<?>> T getObject(Class<T> tipo,String Identifier)
		throws InsideServiceStoreException, InsideStoreObjectNotFoundException 
	{
		
		SortedMap<ObjetoInsideVersion, T> versionesObjeto = getVersionesObjeto(tipo, Identifier);
		
		if(versionesObjeto.size() == 1){
			return versionesObjeto.values().iterator().next();
		}else{
			return versionesObjeto.get(versionesObjeto.lastKey());
		}
	}
	
	private <T extends ObjetoInside<?>> Class<?> getEntityClass(Class<T> tipo) throws InsideServiceStoreException{
		if(tipo == ObjetoExpedienteInside.class){
			return ExpedienteInside.class;
		}else if(tipo == ObjetoDocumentoInside.class){
			return DocumentoInside.class;
		}else{
			throw new InsideServiceStoreException("No se corresponden ningun Entity de BBDD con el tipo " + tipo);
		}
	}
	
	@Override
	public <T extends ObjetoInside<?>> T getObjectVersion(Class<T> tipo,String Identifier, int version)
		throws InsideServiceStoreException,InsideStoreObjectNotFoundException
	{
		SortedMap<ObjetoInsideVersion, T> versionesObjeto = getVersionesObjeto(tipo, Identifier);
		
		for(ObjetoInsideVersion versionInside: versionesObjeto.keySet()){
			if(versionInside.getVersion() == version){
				return versionesObjeto.get(versionInside);
			}
		}
		
		throw new InsideStoreObjectVersionNotFoundException(tipo,Identifier,version);
	}

	@Override
	public <T extends ObjetoInside<K>, K extends ObjetoInsideMetadatos> K getObjectMetadatos(Class<T> tipo, String Identifier)
		throws InsideServiceStoreException,	InsideStoreObjectNotFoundException
	{
		return getObjectMetadatos(tipo, Identifier, 0);
	}

	@Override
	public <T extends ObjetoInside<K>, K extends ObjetoInsideMetadatos> K getObjectMetadatos(Class<T> tipo, String Identifier, int version)
		throws InsideServiceStoreException, InsideStoreObjectNotFoundException
	{
		T objetoInside = null;
		if(version >= 1){			
			objetoInside = getObjectVersion(tipo, Identifier, version);
		}else{
			objetoInside = getObject(tipo, Identifier);
		}
		return objetoInside.getMetadatos();
	}

	@Override
	public <T extends ObjetoInside<?>> T updateObject(T object)
			throws InsideServiceStoreException,
			InsideStoreObjectNotFoundException {
		
		if(object instanceof ObjetoInsideVersionable){
			ObjetoInside<?> stored = this.getObject(object.getClass(), object.getIdentificador());
			((ObjetoInsideVersionable) object).setVersion(ObjetoInsideVersion.createNextVersion(((ObjetoInsideVersionable) stored).getVersion()));
			logger.debug("Subida la versión para objeto de tipo " + object.getClass() + " con identificador " + object.getIdentificador());
		}
		
		persister.persistExpedienteOrDocumento(object);
		
		return object;
	}

	@Override
	public <T extends ObjetoInside<?>> List<ObjetoInsideVersion> getObjectVersions(Class<T> tipo, String Identifier)
		throws InsideServiceStoreException,InsideStoreObjectNotFoundException
	{
		SortedMap<ObjetoInsideVersion, T> versionesObjeto = getVersionesObjeto(tipo, Identifier);
		return new ArrayList<ObjetoInsideVersion>(versionesObjeto.keySet());
	}


	@Override
	public <T extends ObjetoInside<?>> SortedMap<ObjetoInsideVersion, T> getVersionesObjeto(Class<T> tipo, String Identifier)
		throws InsideServiceStoreException,InsideStoreObjectNotFoundException 
	{
		Session session = sessionFactory.openSession();
		
		try {
			SortedMap<ObjetoInsideVersion,T> versionesObjeto = new TreeMap<ObjetoInsideVersion, T>(new ObjetoInsideVersionComparator());
			
			Criteria crit = session.createCriteria(getEntityClass(tipo));
			crit.add(Restrictions.eq( "identificador",Identifier));
			
			List<?> objetos = (List<?>) crit.list();
			
			if(objetos.size() < 1){
				throw new InsideStoreObjectNotFoundException(tipo,Identifier);
			}
			
			for(Object entity : objetos){
				T objetoInside = (T) converter.toInside((E) entity,session);
				
				if(!objetoInside.isVersionable()){
					throw new InsideServiceStoreException("El objeto recuperado no era versionable");
				}
				
				ObjetoInsideVersion version = null;
				
				Method getVersionMethod = ClassUtils.getMethod(tipo, "getVersion");
				try{
					version = (ObjetoInsideVersion) getVersionMethod.invoke(objetoInside);
				}catch (Exception e) {
					throw new InsideServiceStoreException("Error llamanado a getVersion en objeto de tipo " + tipo + " : " + e.getMessage(),e);
				}
				
				versionesObjeto.put(version, objetoInside);
			}
			
			return versionesObjeto;
			
		} catch (InsideServiceStoreException e) {
			throw e;
		}finally{
			session.close();
		}


	}
	

	public SessionFactory getSessionFactory() {
		return sessionFactory;
	}

	@Required
	public void setSessionFactory(SessionFactory sessionFactory) {
		this.sessionFactory = sessionFactory;
	}

	@Override
	public <T extends ObjetoInside<K>, K extends ObjetoInsideMetadatos> ResultadoBusquedaInside<K> queryObjects(ConsultaInside consulta, Class<T> tipo)
		throws InsideServiceStoreException 
	{
		Session session = sessionFactory.openSession();
		
		try {
			Criteria criteria = session.createCriteria(getEntityClass(tipo));
			
			InsideServiceHibernateConverterBusqueda converterBusqueda = new InsideServiceHibernateConverterBusqueda(objectDefinitions);
			
			converterBusqueda.addConsultaInsideToCriteria(criteria,consulta,tipo);
			
			List<?> objetos = null;
			
			criteria.setResultTransformer(Criteria.DISTINCT_ROOT_ENTITY);
			
			try {
				objetos = (List<?>) criteria.list();
			} catch (ClassCastException e) {
				throw new InsideServiceStoreException("Alguno de los datos proporcionados no tenía el tipo de datos adecuado para su columna");
			}
			
			if(tipo == ObjetoExpedienteInside.class){
				List<ItemResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos>> resultados = new ArrayList<ItemResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos>>();	
				
				for(Object entity : objetos){
					ItemResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos> res = new ExpedienteResultadoBusquedaInside();
					
					ObjetoExpedienteInside expedienteInside = (ObjetoExpedienteInside) converter.toInside((E) entity,session);
					
					res.setMetadatos(expedienteInside.getMetadatos());
					res.setVersion(expedienteInside.getVersion());
					res.setIdentificador(expedienteInside.getIdentificador());
					resultados.add(res);
				}
				
				ResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos> resultado = new ResultadoBusquedaInside<ObjetoExpedienteInsideMetadatos>();
				resultado.setResultados(resultados);
				resultado.setPagina(1);
				resultado.setPorPagina(resultados.size());
				resultado.setTotales(resultados.size());
				
				return (ResultadoBusquedaInside<K>) resultado;
				
			}else if(tipo == ObjetoDocumentoInside.class){
				List<ItemResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos>> resultados = new ArrayList<ItemResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos>>();	
				
				for(Object entity : objetos){
					ItemResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos> res = new DocumentoResultadoBusquedaInside();
					
					ObjetoDocumentoInside documentoInside = (ObjetoDocumentoInside) converter.toInside((E) entity,session);
					
					res.setMetadatos(documentoInside.getMetadatos());
					res.setVersion(documentoInside.getVersion());
					res.setIdentificador(documentoInside.getIdentificador());
					resultados.add(res);
				}

				ResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos> resultado = new ResultadoBusquedaInside<ObjetoDocumentoInsideMetadatos>();
				resultado.setResultados(resultados);
				resultado.setPagina(1);
				resultado.setPorPagina(resultados.size());
				resultado.setTotales(resultados.size());
				
				return (ResultadoBusquedaInside<K>) resultado;
				
			}else{
				throw new InsideServiceStoreException("No se como extrarer los resultados para el tipo " + tipo.getName());
			}
			
			
//			List<ItemResultadoBusquedaInside<K>> resultados = new ArrayList<ItemResultadoBusquedaInside<K>>();	
//			
//			for(Object entity : objetos){
//				ItemResultadoBusquedaInside<K> res = null;
//				
//				T objetoInside = (T) converter.toInside((E) entity);
//
//				if(tipo == ObjetoExpedienteInside.class){
//					res = (ItemResultadoBusquedaInside<K>) new ExpedienteResultadoBusquedaInside();
//				}else if(tipo == ObjetoDocumentoInside.class){
//					res = (ItemResultadoBusquedaInside<K>) new DocumentoResultadoBusquedaInside();
//				}
//				res.setMetadatos((K) objetoInside.getMetadatos());
//				res.setVersion(((ObjetoInsideVersionable)objetoInside).getVersion());
//				res.setIdentificador(objetoInside.getIdentificador());
//				resultados.add(res);
//			}
			
			
			
		} catch (InsideServiceStoreException e) {
			throw e;
		}finally{
			session.close();
		}
		
		
	}
	/*
	@Override
	public <T extends ObjetoInside<?>> String generarIdentificador (T object) throws InsideServiceStoreException{
		String identificador = "";
		if (object instanceof ObjetoExpedienteInside) {
			ExpedienteInside exp = queryLastExpedienteInside ();
			Integer sec = 1;
			if (exp != null) {
				sec = exp.getId() + 1;
			}
			 
			identificador = "ES_00010207_2012_EXP_" + getSecuencial(sec.toString());
			
		} else if (object instanceof ObjetoDocumentoInside) {
			DocumentoInside doc = queryLastDocumentoInside ();
			Integer sec = 1;
			if (doc != null) {
				sec = doc.getId() + 1;
			}			
			identificador = "ES_00010207_2012_" + getSecuencial(sec.toString());
		}
		return identificador;
		
	}*/
	
	/**
	 * Devuelve el objeto Inside asociado al nombre de una aplicación.
	 * Si la consulta devuelve más de un resultado dará un error
	 * Si la consulta no devuelve nada devolverá NULL.
	 */
	@Override
	public ObjetoInsideAplicacion getAplicacion (String nombreAplicacion) throws InsideServiceStoreException {
		ObjetoInsideAplicacion aplicacionInside = null;
		Session session = sessionFactory.openSession();
		try {
			Criteria criteria = session.createCriteria(InsideWsAplicacion.class);
			criteria.add(Restrictions.eq("nombre", nombreAplicacion));
			List<?> aplicaciones = (List<?>) criteria.list();
			if (aplicaciones != null && aplicaciones.size() > 0) {
				if (aplicaciones.size() > 1) {
					throw new InsideServiceStoreException("Se ha obtenido más de un resultado para la aplicación " + nombreAplicacion);
				} else {
					InsideWsAplicacion entity = (InsideWsAplicacion) aplicaciones.get(0);
					aplicacionInside = InsideServiceStoreHibernateConverterAplicacion.toInside(entity);					
				}
			}
		} catch (Exception e) {
			throw new InsideServiceStoreException("Excepción obteniendo la aplicacion con nombre " + nombreAplicacion, e);
		} finally {
			session.close();
		}
		
		return aplicacionInside;
	}
	
		
	
	/*private ExpedienteInside queryLastExpedienteInside () throws InsideServiceStoreException {
		Session session = sessionFactory.openSession();
		
		try {
			Criteria criteria = session.createCriteria(ExpedienteInside.class);
			criteria.addOrder( Property.forName("id").desc());
			List<?> lista = criteria.list();
			
			ExpedienteInside entity = null;
			if (lista.size() > 0) {
				entity = (ExpedienteInside )lista.get(0);
			}
			
			return entity;
		} catch (Exception e) {
			throw new InsideServiceStoreException("Excepción obteniendo último objeto de BBDD" ,e);
		} finally {
			session.close();
		}
	
	}
	
	private DocumentoInside queryLastDocumentoInside () throws InsideServiceStoreException {
		Session session = sessionFactory.openSession();
		
		try {
			Criteria criteria = session.createCriteria(DocumentoInside.class);
			criteria.addOrder( Property.forName("id").desc());
			List<?> lista = criteria.list();
			
			DocumentoInside entity = null;
			if (lista.size() > 0) {
				entity = (DocumentoInside) lista.get(0);
			}
			return entity;
		} catch (Exception e) {
			throw new InsideServiceStoreException("Excepción obteniendo último objeto de BBDD" ,e);
		} finally {
			session.close();
		}
	
	}*/
	
	
	/*private String getSecuencial (String id) {
		String ret = id;
		int i=id.length();
		while (i <= 30) {
			ret = "0" + ret;
			i++;
		}
		return ret;
	}*/


	public InsideServiceStoreHibernatePersister<I,E> getPersister() {
		return persister;
	}

	@Required
	public void setPersister(InsideServiceStoreHibernatePersister<I,E> persister) {
		this.persister = persister;
	}


	public InsideServiceStoreHibernateConverter<I,E> getConverter() {
		return converter;
	}


	public void setConverter(InsideServiceStoreHibernateConverter<I,E> converter) {
		this.converter = converter;
	}	
}
